@@ -2,7 +2,7 @@ import itertools
 from collections import defaultdict
 from concurrent.futures import Future, ProcessPoolExecutor, as_completed
 from pathlib import Path
-from typing import Dict, Iterator, List, Set, Tuple, Union
+from typing import Any, Dict, Iterator, List, Set, Tuple, Union
 
 import networkx as nx
 import pandas as pd
@@ -16,7 +16,7 @@ from pyscomotif.data_containers import Residue, Residue_pair_data
 from pyscomotif.index_folders_and_files import index_folder_exists
 from pyscomotif.residue_data_dicts import extract_residue_data
 from pyscomotif.utils import (
-    angle_between_two_vectors,
+    BoundedProcessPoolExecutor, angle_between_two_vectors,
     detect_the_compression_algorithm_used_in_the_index, flatten_iterable,
     get_bin_number, get_sorted_2_tuple, pairwise_euclidean_distance,
     read_compressed_and_pickled_file, sort_and_join_2_strings)
@@ -382,7 +382,7 @@ def run_subgraph_monomorphism(motif_MST: nx.Graph, pairs_of_residues: List[Tuple
     add_resname_as_node_attribute(candidate_PDB_graph)
 
     # NOTE: We don't need to use the edge_matcher functionality of the GraphMatcher because in the previous step we performed a connectivity check,
-    # which ensures that each residue pair of the candidate PDB is in the correct geometrical arrangement AND connected to the correct residue pair. 
+    # which ensures that each residue pair of the candidate PDB is in the correct geometrical arrangement AND connected to a residue pair. 
     # If the connectivity check had not been performed, candidate PDBs with residue pairs in the correct geometrical arrangement BUT connected in
     # the wrong way would pass the monomorphism check given it does not check edge values, only the residue types. This kind of edge case can for example happen
     # when an epitope has a chain of repeating residue pairs (i.e (S,T),(T,S),(S,T), ...).
@@ -467,7 +467,7 @@ def solve_motif_MST(
             )
 
     # Non-parallel version of filter_out_PDBs_with_unconnected_residue_pairs. Using while loop + popitem()
-    # instead of a for loop because it reduces RAM usage.
+    # instead of a for loop to limit RAM usage.
     add_resname_as_node_attribute(motif_MST) # Needed for subgraph monomorphism, see run_subgraph_monomorphism().
     filtered_PDBs_with_all_residue_pairs: Dict[str, List[nx.Graph]] =  {}
     while map_of_PDBs_with_all_residue_pairs:
@@ -493,7 +493,8 @@ def get_PDBs_with_similar_motifs(
     """
     motif_MST_filtered_PDBs_map: Dict[nx.Graph, Dict[str, List[nx.Graph]]] = {}
     n_motifs_to_solve = sum(1 for _ in get_all_motif_MSTs_generator(reference_motif_MST, max_n_mutated_residues, residue_type_policy, motif_residues_data))
-    with ProcessPoolExecutor(max_workers=n_cores) as concurrent_executor:
+
+    with BoundedProcessPoolExecutor(max_workers=n_cores, max_submited_tasks=2*n_cores) as concurrent_executor:
         # Residue-pair level parallelisation
         if n_motifs_to_solve <= n_cores:
             for motif_MST in get_all_motif_MSTs_generator(reference_motif_MST, max_n_mutated_residues, residue_type_policy, motif_residues_data):
@@ -506,12 +507,14 @@ def get_PDBs_with_similar_motifs(
         # Motif level parallelisation. When having to check a large number of motifs as a result of mutated residues provided 
         # by the user, motif level parallelisation is ~35% faster than residue-pair level parallelisation.
         else:
-            submitted_futures = {
-                concurrent_executor.submit(solve_motif_MST, motif_MST, index_folder_path, distance_delta_thr, angle_delta_thr, compression):motif_MST
-                for motif_MST in get_all_motif_MSTs_generator(reference_motif_MST, max_n_mutated_residues, residue_type_policy, motif_residues_data)
-            }
-
             tqdm_progress_bar = get_tqdm_progress_bar(total=n_motifs_to_solve, desc='Searched motifs')
+            submitted_futures: Dict[Future[Any], nx.Graph] = {}
+            for motif_MST in get_all_motif_MSTs_generator(reference_motif_MST, max_n_mutated_residues, residue_type_policy, motif_residues_data):
+                future = concurrent_executor.submit(solve_motif_MST, motif_MST, index_folder_path, distance_delta_thr, angle_delta_thr, compression)
+                future.add_done_callback(lambda _:tqdm_progress_bar.update())
+
+                submitted_futures[future] = motif_MST
+            
             for future in as_completed(submitted_futures):
                 if future.exception():
                     raise future.exception() # type: ignore
@@ -521,8 +524,6 @@ def get_PDBs_with_similar_motifs(
                     motif_MST = submitted_futures[future]
                     motif_MST_filtered_PDBs_map[motif_MST] = filtered_PDBs_with_all_residue_pairs
 
-                tqdm_progress_bar.update()
-
     return motif_MST_filtered_PDBs_map
 
 def search_index_for_PDBs_with_similar_motifs(
