@@ -15,8 +15,16 @@ from pyscomotif.indexing import create_index_folder, update_index_folder
 from pyscomotif.motif_search import search_index_for_PDBs_with_similar_motifs
 from pyscomotif.RMSD_calculation import calculate_RMSD_between_motif_and_similar_motifs
 
+DISCLAIMER = """
+\n
+DISCLAIMER: pyScoMotif is free to use for non-commercial purposes. To use pyScoMotif for commercial purposes, please contact us.
+pyScoMotif is developed by the 3BioInfo group from the ULB, Belgium.
+Reference paper: 'pyScoMotif: Discovery of similar 3D structural motifs across proteins'.
+Main developer: Gabriel Cia.
+\n
+"""
 
-@click.group(help='pyScoMotif: a tool for the discovery of similar 3D structural motifs across proteins.', context_settings={'max_content_width':2000})
+@click.group(help='pyScoMotif: a tool for the discovery of similar 3D structural motifs across proteins.', context_settings={'max_content_width':2000}, epilog=DISCLAIMER)
 def command_line_interface() -> None:
     pass
 
@@ -31,7 +39,8 @@ def check_index_path_option(ctx: Any, param: Any, value: Union[None, Path]) -> P
     
     return default_index_folder_path
 
-@command_line_interface.command()
+
+@command_line_interface.command(epilog=DISCLAIMER)
 @click.argument('database_path', type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True, path_type=Path))
 @click.option('--index_path', default=None, show_default=False, type=click.Path(file_okay=False, dir_okay=True, readable=True, writable=True, path_type=Path), 
               callback=check_index_path_option, help="Full path of the directory that will contain the index files. Defaults to <database_path>/pyScoMotif_index.")
@@ -58,10 +67,11 @@ def create_index(database_path: Path, index_path: Path, pattern: str, compressio
     create_index_folder(database_path, pattern, index_path, compression, n_cores)
     return
 
-@command_line_interface.command()
+
+@command_line_interface.command(epilog=DISCLAIMER)
 @click.argument('database_path', type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True, path_type=Path))
 @click.argument('index_path', type=click.Path(exists=True, file_okay=False, dir_okay=True, writable=True, readable=True, path_type=Path))
-@click.option('--pattern', type=str, default='*.pdb',
+@click.option('--pattern', type=str, default='*.pdb', show_default=True,
                help="File extension pattern of the PDB files in the database that the file detection algorithm should match. Examples: *.pdb, *.pdb.gz, *.ent , etcetc . Note the use of the '*' wildcard. Also note that only simple unix style patterns are accepted, complex regex patterns will fail. To know if your pattern works, test it with pathlib.Path.rglob.")
 @click.option('--n_cores', default=1, show_default=True,
                help='Number of cores to use.')
@@ -110,12 +120,14 @@ def check_residue_type_policy_option(ctx: Any, param: Any, value: str) -> Union[
 
     return position_specific_exchange
 
-@command_line_interface.command()
+@command_line_interface.command(epilog=DISCLAIMER)
 @click.argument('index_path', type=click.Path(exists=True, path_type=Path))
 @click.argument('PDB_file', type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path))
 @click.argument('motif', nargs=-1, callback=check_motif_argument) # -1 => Unlimited number of arguments
 @click.option('--results_output_path', type=click.Path(path_type=Path), default=None, callback=check_results_output_path_option,
               help='Full path of the csv file where the results will be saved (e.g: /home/user/Downloads/similar_motifs.csv). If not given, the results will be saved in the current working directory in a file named with the current timestamp (e.g: pyScoMotif_result_14230214102022.csv)')
+@click.option('--sort_results', is_flag=True, default=True, show_default=True,
+               help="Sort the results output file by number of mutations and RMSD. Set to False if results file is too big and doesn't fit into RAM.")
 @click.option('--residue_type_policy', default='strict', callback=check_residue_type_policy_option,
                help='''Option to control whether mutated versions of the query motif should also be tested. There are 4 possible values: 1) strict (DEFAULT): only similar motifs with strictly identical residues to the query motif are searched. 2) relaxed: finds PDBs with motifs that are similar but with potential mutated residues by allowing residues to mutate according to the residue group they are part of, such as polar, positively charged, aromatic, etcetc (see our Github for the exact grouping of the residues). 3) fully_relaxed: similar to the relaxed mode, but residues can be mutated to any of the 20 possible residues. 4) custom position specific exchange: specify possible residue mutations for the desired positions in your motif using a JSON formated string (e.g: '{"A1":"KL", "A2":"YW", "A5":"E"}'. Note the use of single and double quotes).\n To control the maximum number of simultaneous mutations, see the max_n_mutated_residues option.''')
 @click.option('--max_n_mutated_residues', type=click.IntRange(min=1), default=1, show_default=True,
@@ -131,7 +143,7 @@ def check_residue_type_policy_option(ctx: Any, param: Any, value: str) -> Union[
 @click.option('--n_cores', default=1, show_default=True,
                help='Number of cores to use. Note that using high distance_delta_thr and/or angle_delta_thr values in conjuction with many cores could result in very high RAM usage.')
 def motif_search(
-    index_path: Path, pdb_file: Path, motif: Tuple[str, ...], results_output_path: Path, 
+    index_path: Path, pdb_file: Path, motif: Tuple[str, ...], results_output_path: Path, sort_results: bool,
     residue_type_policy: Union[str, Dict[str,List[str]]], max_n_mutated_residues: int, distance_delta_thr: float, 
     angle_delta_thr: float, rmsd_atoms: str, rmsd_threshold: float, n_cores: int
     ) -> None:
@@ -156,29 +168,21 @@ def motif_search(
         n_cores=n_cores
     )
 
-    pyScoMotif_results_df = calculate_RMSD_between_motif_and_similar_motifs(
+    calculate_RMSD_between_motif_and_similar_motifs(
         motif_MST=motif_MST, 
         PDB_file=pdb_file, 
         PDBs_with_similar_motifs=PDBs_with_similar_motifs, 
         index_folder_path=index_path,
         RMSD_atoms=rmsd_atoms,
         RMSD_threshold=rmsd_threshold,
-        n_cores=n_cores
+        n_cores=n_cores,
+        results_output_path=results_output_path,
+        sort_results=sort_results
     )
 
-    pyScoMotif_results_df.to_csv(results_output_path)
-    if len(pyScoMotif_results_df) == 0:
-        print('No PDB structure with a similar motif was found.')
-
     return
 
 if __name__ == '__main__':
     #scalene_profiler.start()
     command_line_interface(max_content_width=2000) # max_content_width=2000 allows help texts to span the entire width of the terminal
-    #scalene_profiler.stop()
-    
-
-# What if the distance is outside of the available distances, say C_alpha_distance of 25 angstroom in an index made with max 20 angstroom ???
-# Could stop the solve_motif function earlier by checking if the current list of PDBs with all the pairs is empty, in which case we can stop computing the remaining pairs.
-# Currently we don't recalculate the sidechain CMR coordinates of mutated residues in the reference motif, although in theory we should probably do it
-
+    #scalene_profiler.stop()
\ No newline at end of file
